
-- Specifying integer constants for OFFSET and FETCH values
USE AdventureWorks2008R2;
GO
-- Return all rows sorted by the column DepartmentID.
SELECT DepartmentID, Name, GroupName
FROM HumanResources.Department
ORDER BY DepartmentID;

-- 1. Skip the first 5 rows from the sorted result set and return all remaining rows.
SELECT DepartmentID, Name, GroupName
FROM HumanResources.Department
ORDER BY DepartmentID OFFSET 5 ROWS;

-- 2. Skip 0 rows and return only the first 10 rows from the sorted result set.
SELECT DepartmentID, Name, GroupName
FROM HumanResources.Department
ORDER BY DepartmentID 
    OFFSET 0 ROWS
    FETCH NEXT 10 ROWS ONLY;
    
-- 3. Specifying variables for OFFSET and FETCH values   
DECLARE @StartingRowNumber tinyint = 1
      , @FetchRows tinyint = 8;
SELECT DepartmentID, Name, GroupName
FROM HumanResources.Department
ORDER BY DepartmentID ASC 
    OFFSET @StartingRowNumber ROWS FETCH NEXT @FetchRows ROWS ONLY;
GO
    
-- 4. Specifying expressions for OFFSET and FETCH values    
DECLARE @StartingRowNumber tinyint = 1
      , @EndingRowNumber tinyint = 8;
SELECT DepartmentID, Name, GroupName
FROM HumanResources.Department
ORDER BY DepartmentID ASC 
    OFFSET @StartingRowNumber - 1 ROWS 
    FETCH NEXT @EndingRowNumber - @StartingRowNumber + 1 ROWS ONLY
OPTION ( OPTIMIZE FOR (@StartingRowNumber = 1, @EndingRowNumber = 20) );

-- Specifying a constant scalar subquery
USE AdventureWorks2008R2;
GO
CREATE TABLE dbo.AppSettings (AppSettingID int, PageSize int);
GO
INSERT INTO dbo.AppSettings VALUES(1, 10);
GO

-- use appsettings table for fetch size
-- 5. Specifying a constant scalar subquery
DECLARE @StartingRowNumber tinyint = 1;
SELECT DepartmentID, Name, GroupName
FROM HumanResources.Department
ORDER BY DepartmentID ASC 
    OFFSET @StartingRowNumber ROWS 
    FETCH NEXT (SELECT PageSize FROM dbo.AppSettings WHERE AppSettingID = 1) ROWS ONLY;
GO    

-- 6. Fetch non-moving rows, pagesize at a time
-- Ensure the database can support the snapshot isolation level set for the query.
IF (SELECT snapshot_isolation_state FROM sys.databases WHERE name = N'AdventureWorks2008R2') = 0
    ALTER DATABASE AdventureWorks2008R2 SET ALLOW_SNAPSHOT_ISOLATION ON;
GO
-- Set the transaction isolation level for this query to SNAPSHOT.
SET TRANSACTION ISOLATION LEVEL SNAPSHOT;
GO
-- Beging the transaction
BEGIN TRANSACTION;
GO
-- Declare and set the variables for the OFFSET and FETCH values.
DECLARE @StartingRowNumber int = 1
      , @RowCountPerPage int = 3;
-- Create the condition to stop the transaction after all rows have been returned.
WHILE (SELECT COUNT(*) FROM HumanResources.Department) >= @StartingRowNumber
BEGIN
-- Run the query until the stop condition is met.
SELECT DepartmentID, Name, GroupName
FROM HumanResources.Department
ORDER BY DepartmentID ASC 
    OFFSET @StartingRowNumber - 1 ROWS 
    FETCH NEXT @RowCountPerPage ROWS ONLY;
-- Increment @StartingRowNumber value.
SET @StartingRowNumber = @StartingRowNumber + @RowCountPerPage;
CONTINUE
END;
GO
COMMIT TRANSACTION;
GO

-- cleanup
DROP TABLE AppSettings    
GO

USE Northwind
GO

SELECT * FROM 
(
SELECT 
 ROW_NUMBER() OVER (ORDER BY customerid, requireddate) as num,
 customerid, requireddate, orderid
FROM orders
) AS a
WHERE num BETWEEN 401 AND 410;

WITH numbered AS
(
SELECT 
 ROW_NUMBER() OVER (ORDER BY customerid, requireddate) as num,
 customerid, requireddate, orderid
FROM orders
) 
SELECT * 
FROM numbered
WHERE num BETWEEN 401 AND 410;

SELECT customerid, requireddate, orderid
FROM orders
ORDER BY customerid, requireddate 
    OFFSET 400 ROWS
    FETCH NEXT 10 ROWS ONLY;

CREATE INDEX i1 ON orders(customerid, requireddate)

-- execute again, note query plan

DROP INDEX i1 ON orders