
USE tempdb
GO

--Reference:
--CREATE SEQUENCE (Transact-SQL) 
--NEXT VALUE FOR (Transact-SQL) 
--sp_sequence_get_range.

CREATE SCHEMA Test ;
GO

-- 1. Sequences live in schemas
--Creating a sequence that increases by 1
--In the following example, Thierry creates a sequence named CountBy1 that increases by one every time that it is used.
	
CREATE SEQUENCE Test.CountBy1
    START WITH 1
    INCREMENT BY 1 ;
GO

-- 2. Use as default for columns
CREATE TABLE Test.T1
(
  col1 INT PRIMARY KEY DEFAULT NEXT VALUE FOR Test.CountBy1,
  col2 VARCHAR(20) NOT NULL
);

-- 3. Even columns in multiple tables, one sequence
CREATE TABLE Test.T2
(
  col1 INT PRIMARY KEY DEFAULT NEXT VALUE FOR Test.CountBy1,
  col2 VARCHAR(20) NOT NULL
);

-- 4. Or use manually, without default or constraint
CREATE TABLE Test.T3
(
  col1 INT PRIMARY KEY,
  col2 VARCHAR(20) NOT NULL
);

INSERT Test.T3 VALUES(NEXT VALUE FOR Test.CountBy1, 'foo')

-- 5. INCREMENT BY keyword
--Creating a sequence that decreases by 1
--The following example starts at 0 and counts into negative numbers by one every time it is used.
	
CREATE SEQUENCE Test.CountByNeg1
    START WITH 0
    INCREMENT BY -1 ;
GO

-- 6. INCREMENT BY keyword
--Creating a sequence that increases by 5
--The following example creates a sequence that increases by 5 every time it is used.
	
CREATE SEQUENCE Test.CountBy5
    START WITH 5
    INCREMENT BY 5 ;
GO

-- 7. START WITH keyword
--Creating a sequence that starts with a designated number
--After importing a table, Thierry notices that the highest ID number used is 24,328. 
--Thierry needs a sequence that will generate numbers starting at 24,329. The following code creates a sequence that starts with 24,329 and increments by 1.
	
CREATE SEQUENCE Test.ID_Seq
    START WITH 24329
    INCREMENT BY 1 ;
GO

--Creating a sequence using default values
--The following example creates a sequence using the default values.
	
CREATE SEQUENCE Test.TestSequence ;

-- 8. Sequence metadata
--Execute the following statement to view the properties of the sequence.
SELECT * FROM sys.sequences WHERE name = 'TestSequence' ;

--A partial list of the output demonstrates the default values.
--start_value increment mimimum_value maximum_value is_cycling is_cached cache_size current_value 
---2147483648 1         -2147483648   2147483647    0          1         10         -2147483648 

--Creating a sequence with a specific data type
--The following example creates a sequence using the smallint data type, with a range from -32,768 to 32,767.
	
-- 9. Sequence with a different data type
CREATE SEQUENCE SmallSeq
    AS smallint ;

-- 10. Creating a sequence using all arguments
--The following example creates a sequence named DecSeq using the decimal data type, having a range from 0 to 255. The sequence starts with 125 and increments by 25 every time that a number is generated. Because the sequence is configured to cycle when the value exceeds the maximum value of 200, the sequence restarts at the minimum value of 100.
	
CREATE SEQUENCE Test.DecSeq
    AS decimal(3,0) 
    START WITH 125
    INCREMENT BY 25
    MINVALUE 100
    MAXVALUE 200
    CYCLE
    CACHE 3
;

-- 11. Get next value for a sequence
--Execute the following statement to see the first value; the START WITH option of 125.
SELECT NEXT VALUE FOR Test.DecSeq;

--Execute the statement three more times to return 150, 175, and 200.
--Execute the statement again to see how the start value cycles back to the MINVALUE option of 100.
--Execute the following code to confirm the cache size and see the current value.

SELECT cache_size, current_value 
FROM sys.sequences
WHERE name = 'DecSeq' ;
GO

CREATE SEQUENCE Test.RangeSeq
    AS int 
    START WITH 1
    INCREMENT BY 1
    MINVALUE 1
    MAXVALUE 25
    CYCLE
    CACHE 10
;

SELECT name, current_value, * FROM sys.sequences WHERE name = 'RangeSeq' ;

-- return a range of sequence numbers. Consider them "used" in the sequence
DECLARE @range_first_value sql_variant , 
        @range_first_value_output sql_variant ;

EXEC sp_sequence_get_range
@sequence_name = N'Test.RangeSeq'
, @range_size = 4
, @range_first_value = @range_first_value_output OUTPUT ;

SELECT @range_first_value_output AS FirstNumber ;

-- return all possible output values from sp_sequence_get_range
DECLARE  
  @FirstSeqNum sql_variant
, @LastSeqNum sql_variant
, @CycleCount int
, @SeqIncr sql_variant
, @SeqMinVal sql_variant
, @SeqMaxVal sql_variant ;

EXEC sys.sp_sequence_get_range
@sequence_name = N'Test.RangeSeq'
, @range_size = 5
, @range_first_value = @FirstSeqNum OUTPUT 
, @range_last_value = @LastSeqNum OUTPUT 
, @range_cycle_count = @CycleCount OUTPUT
, @sequence_increment = @SeqIncr OUTPUT
, @sequence_min_value = @SeqMinVal OUTPUT
, @sequence_max_value = @SeqMaxVal OUTPUT ;

-- The following statement returns the output values
SELECT
  @FirstSeqNum AS FirstVal
, @LastSeqNum AS LastVal
, @CycleCount AS CycleCount
, @SeqIncr AS SeqIncrement
, @SeqMinVal AS MinSeq
, @SeqMaxVal AS MaxSeq ;

SELECT name, current_value, * FROM sys.sequences WHERE name = 'RangeSeq' ;



















-- 12. Use twice in a SELECT statement
CREATE SEQUENCE dbo.Seq1 AS INT
  START WITH 1
  INCREMENT BY 1

SELECT NEXT VALUE FOR dbo.Seq1 AS v1, NEXT VALUE FOR dbo.Seq1 AS v2;
